@@ -1,6 +1,6 @@
 /* mpn_scan1 -- Scan from a given bit position for the next set bit.
 
-Copyright 1994, 1996, 2001, 2002 Free Software Foundation, Inc.
+Copyright 1994, 1996, 2001, 2002, 2004 Free Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
@@ -16,18 +16,14 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include "gmp.h"
 #include "gmp-impl.h"
 #include "longlong.h"
 
-/* Design issues:
-   1. What if starting_bit is not within U?  Caller's problem?
-   2. Bit index should be 'unsigned'?
-
-   Argument constraints:
+/* Argument constraints:
    1. U must sooner or later have a limb != 0.
  */
 
